#!/usr/bin/perl

#
# HoneyScripts - Very simple and limited Perl scripts for naive honeypots
#
#              - Coded by Arne Vidstrom, http://vidstrom.net/stools/honeyscripts/
#

#
# This program is free software; you can redistribute it and/or modify it under the terms
# of the GNU General Public License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program;
# if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
# MA  02111-1307, USA.
#

use Socket;
use POSIX ":sys_wait_h";

$max_chars = 10000;
$idle_timeout = 30;
$debug_mode = 0;
$log_file = "/var/log/genhoney.log";

$hon_port = $ARGV[0];
select ("STDOUT");
$| = 1;
socket(MASTERSOCKET, PF_INET, SOCK_STREAM, getprotobyname('tcp'));
setsockopt(MASTERSOCKET, SOL_SOCKET, SO_REUSEADDR, 1);
$listen_at = sockaddr_in($hon_port, INADDR_ANY);
bind(MASTERSOCKET, $listen_at) or die "Couldn't bind to port\n";
listen(MASTERSOCKET, 5) or die "Couldn't listen on port\n";
do {
	do {
		$chd = waitpid(-1, WNOHANG);
    } while (-1 != $chd);
	$dummy = accept(CLIENTSOCKET, MASTERSOCKET);
	if ($dummy) {
		$retval = fork();
		if ($retval == 0) {
			&honey;
		} else {
			close(CLIENTSOCKET);
		}
	}
} while ($dummy);
close(MASTERSOCKET);

sub honey {
	$other_end = getpeername(CLIENTSOCKET) or die "Couldn't get client IP\n";
	($port, $iaddr) = unpack_sockaddr_in($other_end);
	$ip_address = inet_ntoa($iaddr);
	if (1 == $debug_mode) {
		print("new client accepted from $ip_address\n");
	}
	$curr_date = localtime;
	$done = 0;
	$line_done = 0;
	do {
		$rin = '';
		vec($rin, fileno(CLIENTSOCKET), 1) = 1;
		$timeout = $idle_timeout;
		$nfound = select($rout = $rin, undef, undef, $timeout);
		if (0 >= $nfound) {
			$done = 1;
		}
		else {
			if (vec($rout, fileno(CLIENTSOCKET), 1)) {
				$maxlen = 1;
				$recvres = recv(CLIENTSOCKET, $data_buffer, $maxlen, 0);
				if (1 == defined($recvres)) {
					$temp = length($data_buffer);
					if (0 == length($data_buffer)) {
						$done = 1;
					}
					else {
						if ($num_chars < $max_chars) {
							$curr_line .= $data_buffer;
							if (-1 != index($curr_line, "\r\n")) {
								$line_done = 1;
							}
							if (-1 != index($curr_line, "\n")) {
								$line_done = 1;
							}
						}
						else {
							$done = 1;
						}
					}
				}
				else {
					$done = 1;
				}
			}
		}
		if (1 == $line_done) {
			while ($curr_line =~ /[\r|\n]$/) {
				chop($curr_line);
			}
			open(LOGFILE, ">>$log_file") or die "Unable to open log file";
			flock(LOGFILE, LOCK_EX);
			$curr_date = localtime;
			print LOGFILE ("From $ip_address to port $hon_port at $curr_date ->\n\n$curr_line\n\n");
			print LOGFILE ("-\n\n");
			flock(LOGFILE, LOCK_UN);
			close(LOGFILE);
			$line_done = 0;
			$curr_line = "";
		}
	} while ($done != 1);

	if (1 == $debug_mode) {
		printf("client disconnected\n");
	}
	close(CLIENTSOCKET);
	exit(0);
}
